/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/



/*
 * Filename: FullReader.cpp
 * Description: Record reader interface
 * Author: Yohanes Nugroho
 * Ported from palmbible plus (http://palmbibleplus.sourceforge.net)
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */
#include "FullReader.h"
#include "Database.h"


FullReader::FullReader(Database *dbase, UInt16 version, UInt16 minRecord, UInt16 maxRecord)
{
        this->maxRecord = maxRecord;
        this->minRecord = minRecord;
        this->dbase = dbase;
        this->version = version;
}

void FullReader::ReadBegin(UInt32 start)
{
        recordIndex = (UInt16) (start / recordSize);
        charIndex = (UInt16) (start % recordSize);
        recordPtr = (UInt8 *)dbase->DmxLockVersionRecord(version, minRecord + recordIndex);
}

void FullReader::ReadEnd()
{
	//dbase->DmxUnlockVersionRecord(version, minRecord + recordIndex);
}

UInt8 FullReader::ReadChar()
{
        if (charIndex >= recordSize){
                recordIndex++;
                charIndex = 0;
                recordPtr = (UInt8 *)dbase->DmxLockVersionRecord(version, minRecord + recordIndex);
        }
        UInt8 resultChar = recordPtr[charIndex];
        charIndex ++;
        return resultChar;
}

FullReader::~FullReader()
{
  for (int i = minRecord; i<=maxRecord; i++){
    dbase->DmxUnlockVersionRecord(version, i);
  }
}
