/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/



/*
 * Filename: Reader.cpp
 * Description: Bible Reader Class
 * Author: Yohanes Nugroho
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */


#ifdef USE_LIB_C
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "usermem.h"
#else
#include <e32base.h>
#include <stdlib.h>
//#include <stdio.h>
#endif

#include "Database.h"
#include "Reader.h"
#include "util.h"


EXPORT_C Reader::Reader()
{
      verse = 0;
	  book=0;
	  chapter=0;
   	version = 0;
   	dbase = 0;
	recordReader = 0;
	version_info = 0;
	pdbaccess = 0;
	totalChapters = 0;
	decoder = NULL;
	recordReader = NULL;
}
#ifdef USE_LIB_C
EXPORT_C void Reader::ConstructL(RFs &fsSession, const char *filename)
#else
EXPORT_C void Reader::ConstructL(RFs &fsSession, const TDesC &filename)
#endif
{
#ifdef USE_LIB_C
	pdbaccess = new PDBAccess();
#else
	pdbaccess = new (ELeave) PDBAccess();
#endif
#ifndef USE_LIB_C
	CleanupStack::PushL(pdbaccess);
#endif
	pdbaccess->ConstructL(fsSession, filename);
#ifdef USE_LIB_C
	dbase = new  Database(pdbaccess);
#else
	dbase = new (ELeave) Database(pdbaccess);
#endif
#ifndef USE_LIB_C
        CleanupStack::PushL(dbase);
#endif
#ifdef USE_LIB_C
	version_info = new  VersionInfo(dbase);
#else
	version_info = new (ELeave) VersionInfo(dbase);
#endif

#ifndef USE_LIB_C
	CleanupStack::Pop(dbase);
	CleanupStack::Pop(pdbaccess);
#endif
}

EXPORT_C Reader::~Reader()
{
	closeAllReader();
	delete version_info;
	delete dbase;
	delete pdbaccess;

}

EXPORT_C void Reader::Bookmark()
{
    bookmarkChapter = chapter;
    bookmarkVerse = verse;
}

EXPORT_C void Reader::Return()
{
        chapter = bookmarkChapter;
        verse = bookmarkVerse;
}

EXPORT_C char *Reader::BookName()
{
	char* status;
	status = (char *)User::AllocL(255);
	status[0] = '\0';
	char *s = version_info->GetComplexBookName(version, this->book);
	if (s) my_strcpy(status, s);
	//my_strcat(status, " ");
	User::Free(s);
	return status;
}

EXPORT_C int Reader::GetSepChar()
{
	return version_info->GetSepChar();
}

EXPORT_C char *Reader::SimpleBookName()
{
	char* status;
	status = (char *)User::AllocL(255);
	status[0] = '\0';
	char *s = version_info->GetSimpleBookName(version, this->book);
	if (s) my_strcpy(status, s);
	//my_strcat(status, " ");
	User::Free(s);
	return status;
}


EXPORT_C bool Reader::SeekVerse(UInt16 version, UInt16 book, UInt16
				chapter, UInt16 verse)
{
        UInt16 totalRecord;

        if (version != this->version || book != this->book){
                //closeAllReader();
                indexRecord = version_info->GetBookRecordIndex(version, book);

                if (indexRecord != 0){
                        totalRecord = version_info->GetBookTotalRecord(version, book);
#ifdef USE_LIB_C
			if (decoder) delete decoder;
                        decoder = new  Decoder(version_info, version);
			decoder->ConstructL();
			if (recordReader) delete recordReader;
                        recordReader = new FullReader(	dbase,
                        				version,
                        				indexRecord + 1,
							indexRecord + totalRecord);

#else
			if (decoder) delete decoder;
			if (recordReader) delete recordReader;
                        decoder = new (ELeave) Decoder(version_info, version);
			decoder->ConstructL();
			//CleanupStack::PushL(decoder);
                        recordReader = new (ELeave) FullReader(	dbase,
                        				version,
                        				indexRecord + 1,
							indexRecord + totalRecord);
#endif
                        this->version = version;
			//CleanupStack::Pop(decoder);
			
                        this->book = book;
                        openIndexRecord();
                } else {
                        this->chapter = 0;
                        this->verse = 0;
                        return false;
                }
        }
        this->chapter = chapter;
        this->verse = verse;

        return true;
}


EXPORT_C bool Reader::SeekVerseInCurrentBook(UInt16 chapter, UInt16 verse)
{
	return SeekVerse(this->version, this->book, chapter, verse);
}

EXPORT_C int Reader::VerseNumber()
{
	return this->verse;
}

EXPORT_C int Reader::ChapterNumber()
{
	return this->chapter;
}

EXPORT_C int Reader::BookNumber()
{
	return this->book;
}


EXPORT_C bool Reader::GetCurrentVerseAsString(char* verse)
{

        if (chapter != 0 && chapter < getTotalChapter() + 1){

		getVerseStartLength();

            if (verseLength > 0){
                 	decoder->DecodeVerseAsString(verse,
                 		recordReader,
                 		verseStart,
                 		verseLength);
                } else {
                        verse[0] = 0;
                }
        } else {
                return false;
        }

        return true;
}


EXPORT_C bool Reader::GetCurrentVerseAsStringF(char* verse,
					       UInt16* linepos,
					       UInt16* lineformat,
					       int* linecount
					       )
{

        if (chapter != 0 && chapter < getTotalChapter() + 1){

		getVerseStartLength();

            if (verseLength > 0){
                 	decoder->DecodeVerseTypeF(verse,
						  linepos,
						  lineformat,
						  linecount,
                 		recordReader,
                 		verseStart,
                 		verseLength);
			if (verse[0]!='\0' && (*linecount)==0) {
			  linepos[*linecount] = 0;
			  lineformat[*linecount] = Decoder::versTextType;
			  (*linecount)++;
			}
                } else {
                        verse[0] = 0;
                }
        } else {
                return false;
        }

        return true;
}


EXPORT_C bool Reader::GetCurrentTextAsString(char* verse, int type)
{

        if (chapter != 0 && chapter < getTotalChapter() + 1){

		getVerseStartLength();

            if (verseLength > 0){
                 	decoder->DecodeVerseType(verse,
                 		recordReader,
                 		verseStart,
                 		verseLength, type);
                } else {
                        verse[0] = 0;
                }
        } else {
                return false;
        }

        return true;
}


EXPORT_C void Reader::MovePrevVerse()
{
        verse --;
        if (verse < 1){
                if (chapter > 1){
                        chapter --;
                        verse = getTotalVerse();
                } else {
					verse = 1;
				}
        }
}

EXPORT_C void Reader::MoveNextVerse()
{
        verse ++;
        if (verse > getTotalVerse()) {
                if (chapter < getTotalChapter()) {
                        chapter ++;
                        verse = 1;
                } else {
                	verse--;
                }
        }
}


int Reader::parseBookName(char *str, char *result)
{
	int i = 0;
	int j = 0;
	int done = 0;
#ifdef USE_LIB_C
	if (strlen(str)<0) return 0;
#else
	if (User::StringLength((const TUint8*)str)<=0) return 0;
#endif

	//get book name until space
	while (str[i] && str[i]!=' ') {
		result[j] = (char)my_tolower(str[i]);
		i++;
		j++;
	}
	
	if (my_isalpha(str[0])) {
		do {
			while (str[i] && str[i]==' ') {
				i++;
			}

			if (!str[i] || my_isdigit(str[i])) {
				done = 1;

			} else {
				result[j] = ' ';
				j++;
				while (str[i] && str[i]!=' ') {
					result[j] = (char)my_tolower(str[i]);
					i++;
					j++;
				}

			}
		} while (!done);
		result[j] = '\0';
		return i;
		//if it starts with a number (eg. 1 john) read next word
	} else if (str[i] && my_isdigit(str[0])) {
		result[j] = str[i];
		i++;j++;
		do {
			while (str[i] && str[i]==' ') {
				i++;
			}
			if (!str[i] || my_isdigit(str[i])) {
				done = 1;
			} else {
				if (result[j-1]!=' ') {
					result[j] = ' ';
					j++;
				}
				while (str[i] && str[i]!=' ') {
					result[j] = (char)my_tolower(str[i]);
					i++;
					j++;
				}
			}
		} while (!done);
		result[j] = 0;
		result[i] = 0;
		return i;
	} else {
		return 0;
	}
}


int Reader::parseNumber(char *str, int &chapter, int &verse)
{
	chapter = 0;
	verse = 0;
	int i = 0;
	int number = 0;
	while (str[i] && str[i]==' ') i++;
	while (str[i] && my_isdigit(str[i])){
		number *= 10;
		number += (str[i]-'0');
		i++;
	}
	chapter = number;
	if (chapter==0) chapter=1;
	verse = 1;
	while (str[i] && !my_isdigit(str[i])) i++;
	if (!str[i]) return i;
	number = 0;
	while (str[i] && my_isdigit(str[i])){
		number *= 10;
		number += (str[i]-'0');
		i++;
	}
	verse = number;
	return i;
}

EXPORT_C void Reader::GetLastVerse(int *chapter, int *verse)
{
	*chapter = getTotalChapter();
	*verse = getTotalVerse(*chapter);
}

EXPORT_C int Reader::GetNextBook()
{
	return version_info->GetNextBook(this->book);
}


EXPORT_C int Reader::GetFirstBook()
{
	return version_info->GetFirstBook();
}

EXPORT_C int Reader::GetPrevBook()
{
	return version_info->GetPrevBook(this->book);
}

EXPORT_C int Reader::GetNextBook(int book)
{
	return version_info->GetNextBook(book);
}


EXPORT_C bool Reader::SeekVerseAsString(UInt16 version, char *str)
{
	/*
	Format:
		<bookname> <chapter> [<verse>]
		<bookname>: [<digit>][<sep>]<string>
	*/
	int i = 0;
#ifdef USE_LIB_C
	int l = strlen(str)+1;
#else
	int l = User::StringLength((const TUint8*)str)+1;
#endif
	char *tmpbookname = (char *)User::AllocL(l);
	Mem::FillZ(tmpbookname, l);

	i = parseBookName(str, tmpbookname);

	if (i==0) {
		User::Free(tmpbookname);
		return false;
	}
	int verse, chapter;
	if (parseNumber(str+i, chapter, verse)==0) {
		chapter = 1;
		verse = 1;
	}

	UInt16 idx = (UInt16)version_info->GetBookIndex(tmpbookname);
	if (idx==0) {
		User::Free(tmpbookname);
		return false;
	}

	//SeekVerse(version, idx, chapter, verse);
	SeekVerse(version, idx, 1, 1);


	int total_chapter = getTotalChapter();
	//if there is only one chapter, such as Jude or 3 John
	//consider Jude 10 to be Jude 1:10
	if (verse==1 && chapter>total_chapter && total_chapter==1) {
		verse = chapter;
		chapter = 1;
	}

	if (chapter>total_chapter) {
		chapter = total_chapter;
		verse = 1;
	}

	if (verse>getTotalVerse(chapter)) verse = getTotalVerse(chapter);
	SeekVerse(version, idx, (UInt16)chapter, (UInt16)verse);

	User::Free(tmpbookname);

	return true;
}

//Private Functions

UInt32 Reader::getValue32Start(UInt32* valuePtr, Int16 index)
{
        UInt32 value;

        if (index > 0){
		char *vptr = (char *)valuePtr;
		unsigned char i1 = vptr[(index - 1)*4];
		unsigned char i2 = vptr[(index - 1)*4+1];
		unsigned char i3 = vptr[(index - 1)*4+2];
		unsigned char i4 = vptr[(index - 1)*4+3];
		UInt32 val = ((UInt32)i1<<24)|((UInt32)i2<<16)|((UInt32)i3<<8)|((UInt32)i4);
                value = val;
        } else {
                value = 0;
        }

        return value;
}

UInt16 Reader::getValue16Start(UInt16* valuePtr, Int16 index)
{
	if (index > 0){
		char *vptr = (char *)valuePtr;
		unsigned char i1 = vptr[(index-1)*2];
		unsigned char i2 = vptr[(index-1)*2+1];
		return ((UInt16)i1<<8)|((UInt16)i2);
	} else {
		return 0;
	}
}

UInt16 Reader::getValue16Length(UInt16* valuePtr, Int16 index)
{
        Int16 diff;

        diff = getValue16Start(valuePtr, index + 1) -
		getValue16Start(valuePtr, index);
        if (diff < 0){
                diff = 0;
        }
        return diff;
}

EXPORT_C UInt16 Reader::getTotalChapter()
{
        return totalChapters;
}

EXPORT_C UInt16 Reader::getTotalVerse()
{
        return getValue16Length(totalVersesAcc, chapter - 1);
}

EXPORT_C UInt16 Reader::getTotalVerse(int chapter)
{
        return (UInt16)getValue16Length(totalVersesAcc, (Int16)chapter - 1);
}

UInt16 Reader::getVerseStartLength()
{
        UInt16 verseAcc;


        verseAcc = getValue16Start(totalVersesAcc, chapter - 1) + verse;
        verseStart = getValue32Start(totalChapterCharsAcc, chapter);
        if (verse > 1){
                verseStart += getValue16Start(totalVerseCharsAcc,
			verseAcc - 1);
                verseLength = getValue16Length(totalVerseCharsAcc,
                        verseAcc - 1);
        } else {
                verseLength = getValue16Start(totalVerseCharsAcc,
                        verseAcc);
		}
	return verseLength;
}

void Reader::openIndexRecord()
{
        UInt16* tempPtr;

        if (version != 0 && indexRecord != 0){
                tempPtr = (UInt16 *)dbase->DmxLockVersionRecord(version, indexRecord);
                totalChapters = fixs(tempPtr[0]);

                totalVersesAcc = &tempPtr[1];
                totalChapterCharsAcc = (UInt32*) &tempPtr[totalChapters + 1];
                totalVerseCharsAcc = &tempPtr[totalChapters * 3 + 1];
        }
}

void Reader::closeIndexRecord()
{
        if (version != 0 && indexRecord != 0){
                dbase->DmxUnlockVersionRecord(version, indexRecord);
        }
}

void Reader::closeAllReader()
{
	if (!dbase) return;
    	if (version != 0 && indexRecord != 0) {
        	closeIndexRecord();
        	delete decoder;
		decoder = NULL;
        	delete recordReader;
		recordReader = NULL;
    	}
}

EXPORT_C char * Reader::GetBookInfo(int book, int &chapter, int &verse)
{
	char *result = version_info->GetComplexBookName(1, (UInt16)book);
	chapter = version_info->GetTotalChapter(1, (UInt16)book);
	verse = 1;
	return result;
}


EXPORT_C int Reader::GetTotalBookChapter(int book)
{
	return version_info->GetTotalChapter(1, (UInt16)book);
}


EXPORT_C int Reader::GetTotalChapterVerse(int book, int chapter)
{
	return version_info->GetTotalVerse(1, (UInt16)book, (UInt16)chapter);
}
