/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/

#include <coecobs.h>
#include <eikenv.h>
#include <eikspane.h>
#include <aknviewappui.h>
#include <aknnotewrappers.h> 
#include <aknlists.h> 

#ifdef __SERIES60_30__
#include <aknsdrawutils.h>
#endif

#include "BookListContainer.h" 
#include "BookListMainView.h"
#include "S60BibleDocument.h"
#include "log.h"

CBookListContainer* CBookListContainer::NewL(CAknView *aView, CS60BibleDocument* aDocument)
{
	CBookListContainer* self = CBookListContainer::NewLC(aView, aDocument);
	CleanupStack::Pop(self);
	return self;
}


CAknSearchField* CBookListContainer::CreateFindBoxL(
	CEikListBox* aListBox,
    CTextListBoxModel* aModel,
    CAknSearchField::TSearchFieldStyle aStyle )
    {
    CAknSearchField* findbox = NULL;

    if ( aListBox && aModel )
        {
        // Gets pointer of CAknFilteredTextListBoxModel.
        CAknFilteredTextListBoxModel* model =
                STATIC_CAST( CAknFilteredTextListBoxModel*, aModel );
        // Creates FindBox.
        findbox = CAknSearchField::NewL( *this,
                                         aStyle,
                                         NULL,
                                         20 );
        CleanupStack::PushL(findbox);
        // Creates CAknListBoxFilterItems class.
        model->CreateFilterL( aListBox, findbox );
        //Filter can get by model->Filter();
        CleanupStack::Pop(findbox); // findbox
        }

    return findbox;
    }


CBookListContainer* CBookListContainer::NewLC(CAknView *aView, CS60BibleDocument* aDocument)
{
	CBookListContainer* self = new (ELeave) CBookListContainer;
	CleanupStack::PushL(self);
	self->ConstructL(aView, aDocument);
	return self;
}


void CBookListContainer::ConstructL(CAknView *aView, CS60BibleDocument* aDocument)
{
	iDocument = aDocument;
	CreateWindowL();
	SetMopParent(aView);
	iListBox = new( ELeave ) CAknDoubleStyleListBox();	
	iTextArray = aDocument->Model()->GetBookList();
	iMatchedArray = new (ELeave) CDesCArrayFlat(66);
	for (int i =0; i<iTextArray->Count(); i++){
		iMatchedArray->AppendL((*iTextArray)[i]);
	}
	
	iListBox->ConstructL( this, EAknListBoxSelectionList);
	iListBox->CreateScrollBarFrameL(ETrue); 
	iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(
		CEikScrollBarFrame::EOn, 
		CEikScrollBarFrame::EAuto );
	iListBox->Model()->SetItemTextArray( iMatchedArray );
	//iTextArray owned by Viewer
	iListBox->Model()->SetOwnershipType( ELbmDoesNotOwnItemArray );	
	int idx = aDocument->Model()->GetBookIndex();
	iListBox->SetCurrentItemIndex(idx);
	TRAPD(err, iListBox->ItemDrawer()->FormattedCellData()->SetSubCellFontL(0, aDocument->Model()->GetFont()));
	TRAPD(err2, iListBox->ItemDrawer()->FormattedCellData()->SetSubCellFontL(1, aDocument->Model()->GetFont()));
	iListBox->HandleItemAdditionL();
	//TInt currentItem = iListBox->CurrentItemIndex();
	TInt currentItem = idx;
        if ( iListBox->ScrollBarFrame() )
        {
                iListBox->ScrollBarFrame()->MoveVertThumbTo( currentItem );
        }
    
    iFindBox = CreateFindBoxL(iListBox, iListBox->Model(), CAknSearchField::EPopup);
	if ( iFindBox )
    {
		// Popup findbox is not shown when listbox display initial.		
		iFindBox->MakeVisible( ETrue );
		iFindBox->SetFocus(ETrue);
    }

	iView = aView;	
	SizeChanged();	
	SetRect(aView->ClientRect());	
	ActivateL();
}

void CBookListContainer::SizeChanged()
{
		if (iListBox) {
			TRect r = Rect();
			r.Resize(0, - iFindBox->Size().iHeight);
			iListBox->SetRect( r ); // Sets rectangle of lstbox.			
		}
        if ( iFindBox )
            {
			AknFind::HandlePopupFindSizeChanged(this, iListBox, iFindBox);
       }
}


#ifdef __SERIES60_30__
void CBookListContainer::HandleResourceChange(TInt aType)
{
	CCoeControl::HandleResourceChange(aType);
    if  ( aType == KEikDynamicLayoutVariantSwitch ) {
    	TRect rect;
    	AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane, rect);    	
		SetRect(rect);
		
    }
	iListBox->DrawNow();
}
#endif


CBookListContainer::~CBookListContainer()
{
	    iFindBox->SetFocus(EFalse);
    	delete iFindBox;	    
    	delete iListBox;
    	delete iMatchedArray;
}

TInt CBookListContainer::CountComponentControls() const
{
    	return 2;
}

CCoeControl* CBookListContainer::ComponentControl(TInt aIndex) const
{
	switch ( aIndex ){
		case 0:
			return iListBox;
		case 1:
			return iFindBox;
		default:
			return NULL;
	}
}

void CBookListContainer::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	gc.SetPenStyle(CGraphicsContext::ENullPen);
	gc.SetBrushColor(KRgbWhite);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);
	
}

void CBookListContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,
    TCoeEvent /*aEventType*/)
{
    // TODO: Add your control event handler code here
}

void CBookListContainer::GotoBook()
{ 
	if (iFindBox) iFindBox->SetFocus(EFalse);
	if (iListBox->CurrentItemIndex()>=0) {
		
        CAknFilteredTextListBoxModel* model =
                STATIC_CAST( CAknFilteredTextListBoxModel*,  iListBox->Model());		
		TPtrC currentText = model->ItemText(iListBox->CurrentItemIndex());		
		int book = 0;
		for (int i=0; i<iTextArray->Count(); i++){
			if (currentText==(*iTextArray)[i]){
				book = i;
				break;
			}
		}	

		iDocument->Model()->GotoBookByIndex(book);
		((CBookListMainView *)iView)->SetRetVal(1);
	}
	iView->HandleCommandL(EAknSoftkeyBack);
}


TKeyResponse CBookListContainer::OfferKeyEventL(
    const TKeyEvent& aKeyEvent,
    TEventCode aType )
{
	if ( aType != EEventKey ) {
        	return EKeyWasNotConsumed;
        }
	if (aKeyEvent.iScanCode == EStdKeyUpArrow) {
        	if (iListBox->CurrentItemIndex()==0) {
        		iListBox->SetCurrentItemIndexAndDraw(iMatchedArray->Count()-1);
        		return EKeyWasConsumed;
        	}
        }
	if (aKeyEvent.iScanCode == EStdKeyDownArrow) {
        	if (iListBox->CurrentItemIndex()==iMatchedArray->Count()-1) {
        		iListBox->SetCurrentItemIndexAndDraw(0);
        		return EKeyWasConsumed;
        	}
        }
        
	if (aKeyEvent.iScanCode == EStdKeyDevice3) {
        	GotoBook();
        	return EKeyWasConsumed;
	}
    if (iListBox) {
        if ( iFindBox )
            {
            TBool needRefresh( EFalse );
			TKeyResponse  r = iFindBox->OfferKeyEventL( aKeyEvent, aType );
			if (r==EKeyWasConsumed) return r;
			/*
            if (CustomFindOfferKeyEventL( aKeyEvent, aType, this, iListBox, iFindBox, ETrue,needRefresh ) == EKeyWasConsumed )
                {
                if ( needRefresh )
                    {
					iListBox->SetCurrentItemIndex(0);
					iListBox->HandleItemAdditionL();
                    SizeChanged();
                    DrawNow();
                    }

                return EKeyWasConsumed;
                }*/
            }
        	return iListBox->OfferKeyEventL( aKeyEvent, aType );
        }
        return EKeyWasNotConsumed;
}

void CBookListContainer::QueryBookList(TDesC& aQueryFrag,TBool aWider)
{		
		_LIT(KTab, "\t");
		if (aWider)
		{			
			iMatchedArray->Delete(0,iMatchedArray->Count());			
			for(TInt j=0;j<iTextArray->Count();j++)
			{
				TPtrC text = ((*iTextArray)[j]);
				int match = text.FindF(aQueryFrag);
				if (match>=0) {
					/*the position of the match should be before the 2nd \t*/
					/*the 1st \t is always on position 1*/					
					TPtrC tstrip = text.Mid(1, text.Length()-2);
					if (tstrip.FindF(KTab)>match)
						iMatchedArray->AppendL((*iTextArray)[j]);
					}
				}
		}
		else
		{
			TInt count = iMatchedArray->Count();

			for (TInt h=0;h<=count-1;h++)
			  {
			  	TPtrC text = (*iMatchedArray)[h];
			  	TPtrC tstrip = text.Mid(1, text.Length()-2);
			  	TPtrC bookname  = tstrip.Mid(0, tstrip.FindF(KTab));
				if (bookname.FindF(aQueryFrag)<0)
					{
						iMatchedArray->Delete(h);
						count--;
						h--;
					}	
			  }
		}
}

TKeyResponse CBookListContainer::CustomFindOfferKeyEventL(const TKeyEvent& aKeyEvent, 
	TEventCode aType, 
	CCoeControl* /*aListBoxParent*/, 
	CEikListBox* /*aListBox*/, 
	CAknSearchField* /*aSearchField*/, 
	TBool /*isFindPopup*/, TBool &aNeedRefresh)
{
	//Note that this method doesn't really provide full editor box
	//functionality, but it's enough for our purposes and allows us to
	//override the behaviour of the search function.

	if ( aType != EEventKey ) // Is not key event?        
        {
        	
        return EKeyWasNotConsumed;
        }

	TBuf<30> k;

	if (aKeyEvent.iCode==EKeyBackspace)
		{
			iFindBox->GetSearchText(k);

			if (k.Length()>=1) k.Delete(k.Length()-1,1);
		
			iFindBox->SetSearchTextL(k);
			QueryBookList(k,ETrue); //Slow step - selection got wider!
			aNeedRefresh=ETrue;
		}

	//LogL(_L("key"), aKeyEvent.iCode);
	
	if (aKeyEvent.iCode==0) return EKeyWasNotConsumed;
	
	TChar charCode(aKeyEvent.iCode);

	iFindBox->GetSearchText(k);
	if (charCode.IsAlphaDigit()||charCode.IsSpace() ) {
		if (k.Length()<30)
		{
			k.Append(charCode);

			iFindBox->SetSearchTextL(k);
			
			QueryBookList(k,EFalse);  //Faster!

			aNeedRefresh=ETrue;
			
		}
	}

	if (aNeedRefresh)
		{
			return EKeyWasConsumed;
		}
	
	return EKeyWasNotConsumed;
}


void CBookListContainer::HandleListBoxEventL(
    CEikListBox* /*aListBox*/,
    TListBoxEvent aEventType )
{
    if ( ( aEventType == MEikListBoxObserver::EEventEnterKeyPressed ) ||
         ( aEventType == MEikListBoxObserver::EEventItemClicked ) || 
         ( aEventType == MEikListBoxObserver::EEventItemDoubleClicked )
         ) {
        	GotoBook();
	}
}
