/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


/* bookmark is not implemented yet */

#include <coemain.h>
#include <eikenv.h>
#include <e32keys.h>
#include <aknutils.h>
#include <e32std.h>
#include <aknnotewrappers.h>
#include <aknviewappui.h>
#include <eiktxlbx.h>
#include <aknpopup.h>
#include <aknlists.h>

#include "BookmarkView.h"
#include "S60BibleUid.h"
#include "S60BibleDocument.h"
#include "S60Bible.hrh"
#include "S60Bible.pan"
#include <S60Bible.rsg>
#include "BookmarkContainer.h"


CBookmarkView* CBookmarkView::NewL(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CBookmarkView* self = CBookmarkView::NewLC(aRect, aDocument);
	CleanupStack::Pop(self);
	return self;
}

TUid CBookmarkView::Id() const
{
	return TUid::Uid(KBookmarkViewID);
}

CBookmarkView* CBookmarkView::NewLC(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CBookmarkView* self = new (ELeave) CBookmarkView();
	CleanupStack::PushL(self);
	self->iDocument = aDocument;
	self->ConstructL(aRect);
	return self;
}

CBookmarkView::CBookmarkView()
{
	//  DoDeactivate is called by the base class destructor
}


CBookmarkView::~CBookmarkView()
{
	//  DoDeactivate is called by the base class destructor
}

void CBookmarkView::ConstructL(const TRect& /*aRect*/)
{
	BaseConstructL(R_S60BIBLE_BOOKMARKVIEW);
}

void  CBookmarkView::DialogDismissedL (  TInt    /*aButtonId*/  )
{
	
}

void CBookmarkView::DoActivateL(const TVwsViewId& aPrevViewId,
                                    TUid /*aCustomMessageId*/,
                                    const TDesC8& /*aCustomMessage*/)
{
	ASSERT(iContainer == NULL);
	iPrevView = aPrevViewId;
	iContainer = CBookmarkContainer::NewL(this, iDocument);
	AppUi()->AddToStackL(iContainer);
}

void CBookmarkView::DoDeactivate()
{
	if (iContainer) {
		AppUi()->RemoveFromStack(iContainer);
		delete iContainer;
		iContainer = NULL;
	}
}

void CBookmarkView::SwitchBack()
{
	AppUi()->ActivateLocalViewL(iPrevView.iViewUid);
}


void CBookmarkView::HandleCommandL(TInt aCommand)
{
    switch(aCommand)	{
        case EAknSoftkeyBack:
        	{
			SwitchBack();
			break;
		}
	case KGotoVerse:
		{
			iContainer->GotoBookmark();
			break;
		}
	case KDelBookmark:
		{
			iContainer->DelBookmark();
			break;
		}
	case KDelAll:
		{
			iContainer->DelAll();
			break;
		}
       default:
		//User::Panic(KS60BiblePanicName, ES60BibleUnknownCommand);
            	break;
	}
}

