/*
* ============================================================================
*  Name     : S80BibleAppUi.cpp
*  Part of  : S80Bible
*  Created  : 22.05.2006 by Yohanes Nugroho
*  Description:
*     CS80BibleAppUi implementation
*  Copyright: 
* ============================================================================
*/

#include <eikenv.h>
#include <ckninfo.h>
#include <eikrted.h>
#include <txtrich.h>
#include <ckntitle.h>
#include <eiktbar.h>
#include <eikaufty.h>
#include <eikspane.h>
#include <eikdoc.h>
#include <bautils.h>

#include "S80Bible.pan"
#include "S80BibleAppUi.h"
#include "S80BibleDocument.h"
#include "S80BibleView.h"
#include "S80Bible.hrh"
#include "S80Bible.rsg"
#include "PDBFilter.h"
#include "gotodialog.h"
#include "recentbibledialog.h"
#include "fontdialog.h"
#include "Viewer.h"
#include "CStringList.h"
#include "listdialog.h"
#include "bookmarkdialog.h"
#include "abouts80.h"
#include "checkbiblepdb.h"

const TInt KDefaultTextGranularity=256;

// ConstructL is called by the application framework
void CS80BibleAppUi::ConstructL()
    {
    BaseConstructL();
	_LIT(KGDR, "*.gdr");
	LoadFontsL(KGDR);
	_LIT(KTTF, "*.ttf");
	LoadFontsL(KTTF);

	TRect r = ClientRect();
    iView = CS80BibleView::NewL(iDocument, r);
	//iEikonEnv->VirtualCursor().SetCursorStateL(TEikVirtualCursor::EOff,*iEikonEnv);
	AddToStackL(iView);

	//TRect clientRect=ClientRect();	
	HBufC *appDll = HBufC::NewLC(KMaxFileName);

	*appDll = iDocument->Application()->AppFullName();
	TParse parsa;
	User::LeaveIfError(parsa.SetNoWild(*appDll, NULL, NULL));
	iDataPath = parsa.DriveAndPath();

	// If the application is in ROM (drive Z:), put settings on C:
	if ( (iDataPath[0] == 'Z') || (iDataPath[0] == 'z') ) {
		iDataPath[0] = 'C';
	}

	iCrashFile = iDataPath;
	iCrashFile.Append(KCrashFile);
		
	iBibleList = iDataPath;
	iBibleList.Append(KBibleListFile);

	iBibleSettings = iDataPath;
	iBibleSettings.Append(KSettingsFile);

	iBibleOptions = iDataPath;
	iBibleOptions.Append(KOptionsFile);

	iDocument->Model()->iOptionsFile = iBibleOptions;

	iDocument->Model()->iBookmarkFile = iDataPath;
	iDocument->Model()->iBookmarkFile.Append(KBookmarkFile);

	iDocument->Model()->iBindingsFile = iDataPath;
	iDocument->Model()->iBindingsFile.Append(KBindingsFile);

	iBibleHistory = iDataPath;
	iBibleHistory.Append(KHistoryFile);

	iRecentBibles = iDataPath;
	iRecentBibles.Append(KRecentFile);

	/*the crash file should not exist*/
	if (BaflUtils::FileExists(CEikonEnv::Static()->FsSession(), iCrashFile)) {
		/*remove settings file to prevent loading same bible file*/
		BaflUtils::DeleteFile(CEikonEnv::Static()->FsSession(), iBibleSettings);		
	}
	
	
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iBibleSettings);

	iHistory = CStringList::NewL(CEikonEnv::Static()->FsSession());
	iHistory->LoadFromFileL(iBibleHistory);

	iRecent = CStringList::NewL(CEikonEnv::Static()->FsSession());
	iRecent->LoadFromFileL(iRecentBibles);


	int book = 10;
	int chapter = 1;
	int verse = 1;
	//ugly, should have used streams instead
	//but sometimes it is usefull to be able to quickly see the content 
	//of the configuration file
	if (settings->Count()){
		TPtrC  str = (*(settings->Content()))[0];
		if (str.Length()>0) {
			CStringList *crashnote = CStringList::NewLC(CEikonEnv::Static()->FsSession());
			crashnote->AddStringL(str);
			crashnote->SaveToFileL(iCrashFile);
			CleanupStack::PopAndDestroy(crashnote);

		if (iDocument->Model()->ChangeBible(str)) {
			if (settings->Count()>5) {
				iLastVerse = (*(settings->Content()))[1];
				book = settings->GetNumberAt(2);
				if (book<=0) book = 10;
				chapter = settings->GetNumberAt(3);
				if (chapter<=0) chapter = 1;
				verse = settings->GetNumberAt(4);
				if (verse<=0) verse = 1;
				int f= settings->GetNumberAt(5);
				if (f>0) iDocument->Model()->setFont(f);
				int e= settings->GetNumberAt(6);
				if (e>0) {
					iDocument->Model()->SetEncoding(e);
				}
			}
			
			if (settings->Count()>12) {
				for (int i = 0; i<6; i++){
					TRgb color(settings->GetNumberAt(7+i));
					iDocument->Model()->SetColor((Viewer::ETextObject)i, color);
				}
			}
	
			
			iDocument->Model()->ClearVerse();
			if (book<0) book = 10;
			if (chapter<0) chapter = 1;
			if (verse<0) verse = 1;
			iDocument->Model()->SeekVerse(1, (UInt16) book, (UInt16)chapter,(UInt16) verse);
			iDocument->Model()->FillVerse();
		}
		}
	}

	CleanupStack::PopAndDestroy(settings);

	LoadSettings(ETrue);
	iDocument->Model()->LoadBookmarkL();

	CleanupStack::PopAndDestroy(appDll); //hbufc
	BaflUtils::DeleteFile(CEikonEnv::Static()->FsSession(), iCrashFile);		
	

    }


void CS80BibleAppUi::LoadSettings(TBool aFirst)
{
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iBibleOptions);
	if (settings->Count()>1){
		if (aFirst) iFullScreen = settings->GetNumberAt(0);
		iBackLight = settings->GetNumberAt(1);
	}
	CleanupStack::PopAndDestroy(settings);
}


CS80BibleAppUi::~CS80BibleAppUi()
    {
	if (iView)
		{
		iEikonEnv->RemoveFromStack(iView);

		delete iView;
	    iView = NULL;
		}

    delete iHistory;
	delete iRecent;
}


CS80BibleAppUi::CS80BibleAppUi(CS80BibleDocument *aDocument) :
	iDocument(aDocument)
    {
    // no implementation required
    }


// Changes normal view to full screen or vice versa. 
void CS80BibleAppUi::ChangeFullScreenL()
	{
	MEikAppUiFactory* factory = iEikonEnv->AppUiFactory(); 
	CEikButtonGroupContainer* bar = factory->ToolBar();			// CBA
	CEikToolBar* band = factory->ToolBand();					// Title
	CEikToolBar* titleband = factory->TitleBand();				// This is not used
	CEikStatusPane* pane = factory->StatusPane();				// left hand status pane
	CCknAppTitle* title	= static_cast<CCknAppTitle*>( band->ControlById(ES80BibleAppTitle) ); // title control

	TInt titleBarAdjacent	= EGulAdjNone;

	if ( titleband )
		{
		titleband->MakeVisible(EFalse);
		}

	bar->SetComponentsToInheritVisibility(ETrue);
	bar->MakeVisible(!bar->IsVisible());
	band->SetComponentsToInheritVisibility(ETrue);


	if ( pane )
		{
		pane->MakeVisible(!pane->IsVisible());
		}

	band->MakeVisible(!band->IsVisible());

	TRect clientRect = ClientRect();

	TRect bandRect	= band->Rect();
	bandRect.iTl.iX	= clientRect.iTl.iX;
	bandRect.iBr.iX	= clientRect.iBr.iX;
	bandRect.iBr.iY	= bandRect.iTl.iY + title->MinimumSize().iHeight;
	band->SetRect(bandRect);
	title->SetRect(band->Rect());

	band->SetAdjacent(titleBarAdjacent);
	band->DrawDeferred();
	bar->DrawDeferred();
	title->DrawDeferred();

	iView->SetRect(clientRect);

	iDocument->Model()->SaveState();

	iDocument->Model()->SetArea(clientRect);
	
	iDocument->Model()->ClearVerse();
	iDocument->Model()->LoadState();
	iDocument->Model()->FillVerse();
	

	iView->DrawDeferred();
	//iEditor->ForceScrollBarUpdateL();
	//iEditor->DrawDeferred();
	}	


TKeyResponse CS80BibleAppUi::HandleKeyEventL(const TKeyEvent& /*aKeyEvent*/, TEventCode /*aType*/)
	{
	//return iEditor->OfferKeyEventL(aKeyEvent, aType);
		return EKeyWasNotConsumed;
	}


void CS80BibleAppUi::SaveSettingsL()
{
		CStringList *settings = CStringList::NewLC(
			CEikonEnv::Static()->FsSession());
		settings->AddStringL(iDocument->Model()->iCurrentBook);
		settings->AddStringL(iLastVerse);
		settings->AddNumberL(iDocument->Model()->GetBookNumber());
		settings->AddNumberL(iDocument->Model()->GetStartChapter());
		settings->AddNumberL(iDocument->Model()->GetStartVerse());
		settings->AddNumberL(iDocument->Model()->GetFontNum());
		settings->AddNumberL(iDocument->Model()->GetEncoding());
		for (int i = 0; i<6; i++){
			TRgb color = iDocument->Model()->GetColor((Viewer::ETextObject)i);
			settings->AddNumberL(color.Value());
		}
		settings->SaveToFileL(iBibleSettings);
		CleanupStack::PopAndDestroy(settings);
		iHistory->SaveToFileL(iBibleHistory);
}


void CS80BibleAppUi::DoOpen()
{
		TFileName aFileName;
		//TBuf<5> ext(_L(".ext"));
		PDBFilter myPDBFilter;
		if (!CCknOpenFileDialog::RunDlgLD(aFileName, &myPDBFilter, _L("PDB"))) {
			iView->SetFocus(ETrue, EDrawNow);
			iView->DrawDeferred();
			return;
		}
		if (!CheckBiblePDB::CheckFile(CEikonEnv::Static()->FsSession(), aFileName)) {
			CCknInfoDialog::RunDlgLD(_L("Error"), _L("Not a valid PDB Bible file"));
			iView->DrawDeferred();
			return;
		}

		if (iDocument->Model()->ChangeBibleL(aFileName)) {
			iDocument->Model()->FillVerse();
			iDocument->Model()->ClearBookList();
			iView->SetFocus(ETrue, EDrawNow);
			
			CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
			settings->AddStringL(aFileName);
			settings->AddStringL(iLastVerse);
			settings->SaveToFileL(iBibleSettings);
			CleanupStack::PopAndDestroy(settings);

			TBool found = EFalse;
			for (int i = 0; i<iRecent->Count(); i++){
				if (((*(iRecent->Content()))[i]).CompareF(aFileName)==0) {
					found = ETrue;
				}
			}
			if (!found) iRecent->PrependL(aFileName);
			iRecent->SaveToFileL(iRecentBibles);

		}
	iView->DrawDeferred();

}


void CS80BibleAppUi::DoGoto()
{

	if (!iDocument->Model()->IsReady()) return;
	
	
	int i;
	CGotoDialog *db = new (ELeave)CGotoDialog(selBook, selChapter, selVerse);
	CDesCArrayFlat * strs = new CDesCArrayFlat(100);
	//selChapter.SetLength(0);
	//selVerse.SetLength(0);

	CDesCArray *books = iDocument->Model()->GetBookList();
	for (i=0; i<books->Count(); i++){
		strs->AppendL((*books)[i]);
	}

	db->SetList(strs);

	if (db->ExecuteLD(R_GOTO)) {
		//iDocument->Model()->GotoBookByIndex(selBook);
		int chapter = 0;
		TLex t(selChapter);
		t.Val(chapter);
		int verse = 0;
		TLex t2(selVerse);
		t2.Val(verse);
		if (chapter>0 && verse<=0) {
			verse = 1;
		}

		TBuf<128> bookname;
		//iDocument->Model()->MyStringToDes(iDocument->Model()->BookName(), bookname);
		bookname = (*strs)[selBook];
		iLastVerse.SetLength(0);
		iLastVerse.Append(bookname);
		iLastVerse.Append(_L(" "));

		if (chapter>0 && verse>0) {
				iLastVerse.AppendNum(chapter);
				iLastVerse.Append(_L(" "));
				iLastVerse.AppendNum(verse);
		}else {
			iLastVerse.Append(_L("1 1"));
		}
		char buff[256];
		iDocument->Model()->MyDesToString(iLastVerse, buff);

		if (iDocument->Model()->SeekVerseAsString(1, buff)){
				iDocument->Model()->ClearVerse();
				iDocument->Model()->FillVerse();				
				TBool found = EFalse;
				for (int i = 0; i<iHistory->Count(); i++){
					if (((*(iHistory->Content()))[i])==iLastVerse) {
						found = ETrue;
					}
				}
				if (!found) iHistory->PrependL(iLastVerse);
				if (iHistory->Count()>20) {
					iHistory->Delete(20);
				}
				iHistory->SaveToFileL(iBibleHistory);

		}
		
	}
	delete strs;
	iView->DrawDeferred();
	iDocument->Model()->ClearBookList();

}

void CS80BibleAppUi::LoadFontsL(const TDesC& aExtension)
{
	HBufC* appName = iDocument->Application()->AppFullName().AllocLC();                    
    TParsePtrC parse(*appName);                                                 
                                                                                
    // Construct search path                                                                                                    
    TFileName file;                                                             
    file.Copy(parse.DriveAndPath()); 
    file.Append(aExtension);                                                        
    RFs& fs = iCoeEnv->FsSession();                                             
                                                                                
    CDir* entries;                                                              
    fs.GetDir(file, KEntryAttNormal, ESortByName, entries);                     
    CleanupStack::PushL(entries); 
    for (TInt i = 0; i < entries->Count(); ++i)                                 
	{
		TInt id = 0;
		tmpFontPath.Copy(parse.DriveAndPath());
		tmpFontPath.Append((*entries)[i].iName);
		CEikonEnv::Static()->ScreenDevice()->AddFile(tmpFontPath, id);
	}
	CleanupStack::PopAndDestroy(entries);
	CleanupStack::PopAndDestroy(appName);

}


void CS80BibleAppUi::DoFontSelect()
{
	CWsScreenDevice* dev = iCoeEnv->ScreenDevice();
        CDesCArrayFlat * selection = new (ELeave) CDesCArrayFlat(2);

	for ( TInt t = 0; t < dev->NumTypefaces(); t++ ) {
		TTypefaceSupport typefaceSupport;
		dev->TypefaceSupport(typefaceSupport, t);
		selection->AppendL(typefaceSupport.iTypeface.iName.Des());
	}
	int index = 0;
	CFontDialog *fl = new 	(ELeave)CFontDialog (index);
	fl->SetList(selection);
	if (fl->ExecuteLD(R_FONT)) {
		iDocument->Model()->setFont(index);
		iDocument->Model()->ClearVerse();
		iDocument->Model()->FillVerse();		
	}
	iView->DrawDeferred();
}

void CS80BibleAppUi::DoGotoRecentBible()
{

	if (iRecent->Count()==0) {
		DoOpen();
		return;
	}

	int idx = 0, i;
	CRecentBibleDialog * rb = new (ELeave) CRecentBibleDialog(idx);
	CDesCArrayFlat * strs = new CDesCArrayFlat(100);
	for (i=0; i<iRecent->Count(); i++){
		strs->AppendL((*(iRecent->Content()))[i]);
		
	}

	rb->SetList(strs);

	if (rb->ExecuteLD(R_RECENT_BIBLE)) {
		if (iDocument->Model()->ChangeBibleL((*(iRecent->Content()))[idx])) {
			iDocument->Model()->FillVerse();
			iDocument->Model()->ClearBookList();
			iView->SetFocus(ETrue, EDrawNow);
		}
	}

	
	iView->DrawDeferred();
}

CStringList *CS80BibleAppUi::GetRecentBible()
{
	return iRecent;
}


void CS80BibleAppUi::DoHistory()
{
	int idx = 0, i;
	CListDialog * rb = new (ELeave) CListDialog (idx);
	CDesCArrayFlat * strs = new CDesCArrayFlat(100);
	for (i=0; i<iHistory->Count(); i++){
		
		strs->AppendL((*(iHistory->Content()))[i]);
		
	}

	rb->SetList(strs);

	if (rb->ExecuteLD(R_BIBLE_HISTORY)) {
		char buff[256];
		iDocument->Model()->MyDesToString((*(iHistory->Content()))[idx], buff);
		if (iDocument->Model()->SeekVerseAsString(1, buff)){
				iDocument->Model()->ClearVerse();
				iDocument->Model()->FillVerse();
				iView->SetFocus(ETrue, EDrawNow);
		}
	}
	
	iView->DrawDeferred();
	
}


void CS80BibleAppUi::ViewBookmarks()
{
	int idx = 0;
	CBookmarkDialog * rb = new (ELeave) CBookmarkDialog (idx);

	

	rb->SetList(iDocument->Model()->GetBookmark());


	if (rb->ExecuteLD(R_BOOKMARKS)) {

		if (iDocument->Model()->GetBookmark()->Count()>0)  {
			if (idx==-1) return;
			CDesCArrayFlat * f = (CDesCArrayFlat * )iDocument->Model()->GetBookmark();
	
			TBuf<100> verse;
			verse = (*f)[idx];
			char buff[100];
			Mem::Fill(buff, sizeof buff, 0);
			for(int i=1; i<verse.Length(); i++){
				if (((char)verse[i])=='\t') break;
				buff[i-1] = (char)verse[i];
			}
			if (iDocument->Model()->SeekVerseAsString(1, buff)) {
				iDocument->Model()->ClearVerse();
				iDocument->Model()->FillVerse();
			} 
		}
	}
	iView->DrawDeferred();
}


// handle any menu commands
void CS80BibleAppUi::HandleCommandL(TInt aCommand)
	{
    switch(aCommand)
		{
		case EEikCmdExit:
			{
				SaveSettingsL();
			CBaActiveScheduler::Exit();
			}
			break;
		case ES80BibleFontSelect:
			{
				DoFontSelect();

			}
			break;

		case ES80BibleCopy:
			{
				if (iDocument->Model()->IsReady()){
					iDocument->Model()->CopyToClipboardL(1);
					User::InfoPrint(_L("Verse Copied"));
				}
				break;
			
			}
		case ES80BibleFullScreen:
			{
			ChangeFullScreenL();
			}
			break;
		case ES80BibleOpen:
			{
				DoOpen();
			}
			break;
		case ES80BibleGoto:
			{
					DoGoto();
			}
			break;
		case EGotoRecentBible:
			{
					DoGotoRecentBible();
			}
			break;
		case EHistory:
			{
				DoHistory();
			}
			break;
		case EAddBookmark:
			{
				iDocument->Model()->AddToBookmark();
				iDocument->Model()->SaveBookmarkL();
				User::InfoPrint(_L("Bookmark Added"));				
			}
			break;
		case EBookmarks:
			{
				ViewBookmarks();
			}
			break;
		case EAbout:
			{
				//CCknInfoDialog::RunDlgLD(_L("About"), _L("Copyright 2006 Yohanes Nugroho"));
				CAboutDialog * rb = new (ELeave) CAboutDialog();
				rb->ExecuteLD(R_ABOUT);
				iView->DrawDeferred();
			}
			break;
	case KPrevLine:
	case KNextLine:
	case KPrevPage:
	case KNextPage:
	case KNextVerse:
	case KPrevVerse:
	case KNextChapter:
	case KPrevChapter:
	case KGotoTop:
	case KGotoBottom:
	case KNextBook:
	case KPrevBook:
		{
			iView->HandleCommandL(aCommand);
			break;
		}
	


        default:
			{
            //Panic(ES80BibleUi);
            break;
			}
        }
    }


TBool CS80BibleAppUi::ProcessCommandParametersL( TApaCommand /*aCommand*/, TFileName& /*aDocumentName*/, const TDesC8& /*aTail*/)
	{
	// TODO: Implement ProcessCommandParametersL function to enable processing shell commands.
	return EFalse;
	}

CS80BibleDocument *CS80BibleAppUi::Document()
{
	return iDocument;
}