// INCLUDE FILES
#include <eikappui.h>
#include <eikenv.h>
#include <eikon.hrh>


#include <S80bible.rsg>
#include "S80BibleView.h"
#include "S80BibleDocument.h" 
#include "Viewer.h" 
#include "VerseInfo.h" 
#include "S80Bible.hrh"
#include "bitmapmethods.h"
#include "Decoder.h"
// ================= MEMBER FUNCTIONS =========================================

// ----------------------------------------------------------------------------
// S80BibleView::S80BibleView()
// Default constructor.
// ----------------------------------------------------------------------------
//
/*
CS80BibleView::CS80BibleView()
:   iContainer( NULL )
    {
    }
*/

CS80BibleView* CS80BibleView::NewL(CS80BibleDocument *aDocument, TRect &aRect)
	{
	CS80BibleView* self = new (ELeave) CS80BibleView();
	self->iDocument = aDocument;
	CleanupStack::PushL(self);
	self->ConstructL(aRect);
	CleanupStack::Pop(self);	// self
	return self;
	}

void CS80BibleView::ConstructL(const TRect& aRect)
	{
	iRect = aRect;
	iDocument->Model()->SetArea(aRect);
	CreateWindowL();
	    // Set the windows size
    SetRect(aRect);

	delete iOffScreenBitmapGc;
	iOffScreenBitmapGc = NULL;

	delete iOffScreenBitmapDevice;
	iOffScreenBitmapDevice = NULL;

	delete iOffScreenBitmap;
	iOffScreenBitmap = NULL;


	iOffScreenBitmap = NBitmapMethods::CreateBitmapL(Rect().Size(),KColourDepth);
	iOffScreenBitmapDevice = NBitmapMethods::CreateBitmapDeviceL(*iOffScreenBitmap);
	iOffScreenBitmapGc = NBitmapMethods::CreateGraphicsContextL(*iOffScreenBitmapDevice);


	//SetBlank();
	EnableDragEvents();

	ActivateL();
	}


CS80BibleView::CS80BibleView()
{

}

// ----------------------------------------------------------------------------
// S80BibleView::~S80BibleView()
// Destructor.
// ----------------------------------------------------------------------------
//
CS80BibleView::~CS80BibleView()
    {
	//DoDeactivate();
    //CloseContainer(); // Deletes container object.
	delete iOffScreenBitmapGc;
	iOffScreenBitmapGc = NULL;

	delete iOffScreenBitmapDevice;
	iOffScreenBitmapDevice = NULL;

	delete iOffScreenBitmap;
	iOffScreenBitmap = NULL;

    }

// ----------------------------------------------------------------------------
// TUid S80BibleView::Id()
// Returns ID of view.
// ----------------------------------------------------------------------------
//
TUid CS80BibleView::Id() const
    {
	    return KViewId;
    }

// ----------------------------------------------------------------------------
// S80BibleView::HandleCommandL( TInt )
// Handles the commands.
// ----------------------------------------------------------------------------
//
void CS80BibleView::HandleCommandL( TInt aCommand )
    {
        //iContainer->HandleCommandL( aCommand );
	switch(aCommand) {

	case KPrevLine:
			{
				iDocument->Model()->ScrollUp(1);
				iDocument->Model()->UpdateBuffer();
				DrawDeferred();
				break;
			}
	case KNextLine:
			{
				iDocument->Model()->ScrollDown(1);
				iDocument->Model()->UpdateBuffer();
				DrawDeferred();
				break;
			}
	case KPrevPage:
			{
				iDocument->Model()->PageUp();
				DrawDeferred();
				break;
			}
	case KNextPage:
			{
				iDocument->Model()->PageDown();
				DrawDeferred();
				break;
			}
	case KNextVerse:
			{
				iDocument->Model()->NextVerse();
				DrawDeferred();
				break;
			}
	case KPrevVerse:
			{
				iDocument->Model()->PrevVerse();
				DrawDeferred();
				break;
			}
	case KNextChapter:
			{
				iDocument->Model()->NextChapter();
				DrawDeferred();
				break;
			}
	case KPrevChapter:
			{
				iDocument->Model()->PrevChapter();
				DrawDeferred();
				break;
			}
	case KGotoTop:
			{
				iDocument->Model()->GotoTop();
				DrawDeferred();
				break;
			}
	case KGotoBottom:
			{
				iDocument->Model()->GotoBottom();
				DrawDeferred();
				break;
			}
	case KNextBook:
			{
				iDocument->Model()->NextBook();
				DrawDeferred();
				break;
			}
	case KPrevBook:
			{
				iDocument->Model()->PrevBook();
				DrawDeferred();
				break;
			}
	}
}

//DEL void CS80BibleView::HandlePointerEventL(const TPointerEvent& aPointerEvent)
//DEL {
//DEL 	if (aPointerEvent.iType == TPointerEvent::EButton1Down) {
//DEL 				iDocument->Model()->PageDown();
//DEL 				DrawDeferred();
//DEL 	}
//DEL 
//DEL 	if (aPointerEvent.iType == TPointerEvent::EButton1Up) {
//DEL 				iDocument->Model()->PageUp();
//DEL 				DrawDeferred();
//DEL 
//DEL 	}
//DEL 	//User::InfoPrint(_L("Event"));
//DEL }



// ----------------------------------------------------------------------------
// S80BibleView::CloseContainer()
// Deletes container class object.
// ----------------------------------------------------------------------------
//
void CS80BibleView::CloseContainer()
    {
    }

// ----------------------------------------------------------------------------
// S80BibleView::DoActivateL()
// Creates container class object.
// ----------------------------------------------------------------------------
//
//DEL void CS80BibleView::DoActivateL()
//DEL     {
//DEL     }

// ----------------------------------------------------------------------------
// S80BibleView::DoDeactivate()
// Deletes container class object.
// ----------------------------------------------------------------------------
//
//DEL void CS80BibleView::DoDeactivate()
//DEL     {
//DEL     }

// ----------------------------------------------------------------------------
// void S80BibleView::SetTitleL(const TInt aResouceId)
// Set Title and Show Outline Number
// ----------------------------------------------------------------------------
//
//DEL void CS80BibleView::SetTitleL(const TInt aResouceId)
//DEL     {
//DEL     TBuf<128> MessageText;
//DEL     iCoeEnv->ReadResource( MessageText, aResouceId );
//DEL     }

// End of File



void CS80BibleView::Draw(const TRect& aRect) const
	{

	CWindowGc& windowgc = SystemGc();
	CFbsBitGc *graphicsContextx = iOffScreenBitmapGc;
	CFbsBitGc &graphicsContext = *graphicsContextx;
	
	TRect drawRect=Rect();

	//graphicsContext->CancelClippingRegion();
	//graphicsContext->CancelClippingRect();
	graphicsContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	graphicsContext.SetBrushColor(iDocument->Model()->GetColor(Viewer::EVerseBackground));
	//graphicsContext.SetBrushColor(KRgbBlue);

	graphicsContext.Clear();
	
	if (!iDocument->Model()->IsReady()){
		HBufC* str = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_BIBLE_NOT_FOUND);
		//_LIT(msg, "Bible Not Found");
		graphicsContext.UseFont(CEikonEnv::Static()->NormalFont());
		int w = CEikonEnv::Static()->NormalFont()->TextWidthInPixels(*str);
		int ctr = (drawRect.Width()-w)/2;
		graphicsContext.DrawText(*str,  TPoint(ctr, (drawRect.Height()/2)));
		CleanupStack::PopAndDestroy(str);
		graphicsContext.DiscardFont();
		return;
	}
	
	const CFont *font = iDocument->Model()->GetFont();
	char *s = NULL;
	if (iDocument->Model()->UseLongBookName()) {
		s = iDocument->Model()->BookName();
	} else {
		s = iDocument->Model()->SimpleBookName();
	}
	TBuf<256> stat;
	iDocument->Model()->MyStringToDes(s, stat);
	
	User::Free(s);
	
	VerseInfo *v = iDocument->Model()->GetVerseHead();
	
	int start_chapter, start_verse, start_line;
	start_chapter = iDocument->Model()->GetStartChapter();
	start_verse = iDocument->Model()->GetStartVerse();
	start_line = iDocument->Model()->GetStartLine();
	
	
	while (v) {
		if (v->verse==start_verse && v->chapter==start_chapter) {
			break;
		}
		v = v->next;
	}
 

	int EOrientationNormal = 0;
	int EOrientationLandscape90 = 1;
	int EOrientationLandscape270 = 2;
	int iOrientation = EOrientationLandscape270;
	iOrientation=0;
	TBool up = iOrientation==EOrientationLandscape90;	
	
	
	TBuf<4> chapnum;
	chapnum.Zero();
	chapnum.AppendNum(start_chapter);
	
	graphicsContext.UseFont(font);
	
	TPoint start;
	if (iOrientation==EOrientationNormal) {		
	 start.SetXY(15 , 20 + font->HeightInPixels());
	} else {
		if (up) {
	 		start.SetXY(20 + font->HeightInPixels(), drawRect.Height()-15);	
		} else {
			start.SetXY(20 + font->HeightInPixels(), 15);	
		}
	}
	graphicsContext.UseFont(font);
	
	TRect frame;
	
	if (iOrientation==EOrientationNormal) {	
		frame.SetRect(0, 0, drawRect.Width(), 20 + font->HeightInPixels());
	} else {
		if (up) {
			frame.SetRect(0, 0, 20 + font->HeightInPixels(), drawRect.Height());
		} else {
			frame.SetRect(drawRect.Width()-20 - font->HeightInPixels(), 0, drawRect.Width(), drawRect.Height());
			
		}
	}
	graphicsContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	graphicsContext.SetBrushColor(iDocument->Model()->GetColor(Viewer::EHeaderBG));
	graphicsContext.SetPenColor(iDocument->Model()->GetColor(Viewer::EHeaderFrame));
	graphicsContext.DrawRect(frame);
	frame.Shrink(1,1);
	graphicsContext.DrawRect(frame);
	graphicsContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	graphicsContext.SetPenColor(iDocument->Model()->GetColor(Viewer::EHeaderText));
	
	int chap_draw_x = font->TextWidthInPixels(stat) + font->MaxCharWidthInPixels();
	
	if (iOrientation==EOrientationNormal) {	
		graphicsContext.DrawText(stat, TPoint(10, font->HeightInPixels()+5));
		graphicsContext.DrawText(chapnum, TPoint(chap_draw_x, font->HeightInPixels()+5));
	} else {
		if (up) {
			graphicsContext.DrawTextVertical(stat, TPoint(font->HeightInPixels()+5,
				drawRect.Height()-10), up);
			graphicsContext.DrawTextVertical(stat, TPoint(font->HeightInPixels()+5,
				drawRect.Height()-chap_draw_x), up);
			
		} else {
			graphicsContext.DrawTextVertical(stat, TPoint(drawRect.Width()-(font->HeightInPixels()+5),
				10), up);
			graphicsContext.DrawTextVertical(stat, TPoint(drawRect.Width()-(font->HeightInPixels()+5),
				chap_draw_x), up);
				
		}	
	}
	if (iOrientation==EOrientationNormal) {	
		start.iY += font->HeightInPixels(); 
	} else {
		if (up) {
			start.iX += font->HeightInPixels();
		} else {
			start.iX += font->HeightInPixels();			
			start.iX = drawRect.Width()-start.iX;
		}
	}
	TBuf<4> strversenum;
	TRgb textColor = iDocument->Model()->GetColor(Viewer::EVerseText);
	TRgb numberColor = iDocument->Model()->GetColor(Viewer::EVerseNumber);
	graphicsContext.SetPenColor(textColor);
	//Normal, Landscape90, Landscape270
	int starttext = up?drawRect.Height():2;
	int iidx = start_line;
	if (!iDocument->Model()->IsRToL()){ 
		while (v) {
			graphicsContext.SetPenColor(textColor);
			int inlimit = 1;
			if (iOrientation==EOrientationNormal) {				
					inlimit = start.iY<drawRect.Height();
			} else {
					if (up) {
						inlimit = start.iX<drawRect.Width();
					} else {
						inlimit = start.iX>0;
					}
			}
			/*Draw Verses*/
			while (iidx<v->lines->Count() && inlimit) {

				if (iidx == v->verseline) {

					strversenum.Zero();
					strversenum.AppendNum(v->verse);
					int fw = font->TextWidthInPixels(strversenum);
					int sw = fw+5;
					graphicsContext.SetPenColor(numberColor);
					if (iOrientation==EOrientationNormal) {
						if (fw==0) {
							graphicsContext.DiscardFont();
							graphicsContext.UseFont(CEikonEnv::Static()->NormalFont());
							sw = CEikonEnv::Static()->NormalFont()->TextWidthInPixels(strversenum)+5;
						}							
						graphicsContext.DrawText(strversenum,  TPoint(2, start.iY));
						if (fw==0) {
							graphicsContext.DiscardFont();
							graphicsContext.UseFont(font);
						}						
						graphicsContext.SetPenColor(textColor);	

						graphicsContext.DrawText(v->lines->At(iidx), TPoint(sw, start.iY));

					} else {

						int yp = up?drawRect.Height()-2:2;
						graphicsContext.DrawTextVertical(strversenum,  TPoint(start.iX, yp), up);
						graphicsContext.SetPenColor(textColor);
						yp = up?drawRect.Height()-sw:sw;
						graphicsContext.DrawTextVertical(v->lines->At(iidx), TPoint(start.iX, yp), up);
					}
				} else {
					
					if (v->lineformat->At(iidx)==Decoder::versTextType) {
						graphicsContext.SetPenColor(textColor);
					} else {
						graphicsContext.SetPenColor(iDocument->Model()->GetColor(Viewer::EHeaderText));
					}
					if (iOrientation==EOrientationNormal) {
						graphicsContext.DrawText((TPtrC16)v->lines->At(iidx), TPoint(2, start.iY));
					} else {
						graphicsContext.DrawTextVertical((TPtrC16)v->lines->At(iidx), TPoint(start.iX, starttext), up);					
					}
				}
				if (iOrientation==EOrientationNormal) {				
					start.iY += font->HeightInPixels();
					inlimit = start.iY<drawRect.Height();
				} else {
						if (up) {
							start.iX += font->HeightInPixels();
							inlimit = start.iX<drawRect.Width();
						} else {
							start.iX -= font->HeightInPixels();
							inlimit = start.iX>0;
						}
				}
				iidx++;
			}
			iidx = 0;
			v = v->next;
		}
	} else { /*Right to left*/
		while (v) {
			graphicsContext.SetPenColor(textColor);
			int inlimit = 1;
				if (iOrientation==EOrientationNormal) {				
					inlimit = start.iY<drawRect.Height();
				} else {
						if (up) {
							inlimit = start.iX<drawRect.Width();
						} else {
							inlimit = start.iX>0;
						}
				}
			/*Draw Verses RTOL*/
			while (iidx<v->lines->Count() && inlimit) {
				if (iidx == v->verseline) {
					strversenum.Zero();
					strversenum.AppendNum(v->verse);
					graphicsContext.SetPenColor(numberColor);
					int sw = font->TextWidthInPixels(strversenum);
					int wl = font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));	
					if (iOrientation==EOrientationNormal) {
						graphicsContext.DrawText(strversenum,  TPoint(drawRect.Width()-sw, start.iY));
						graphicsContext.SetPenColor(textColor);
						graphicsContext.DrawText(v->lines->At(iidx), TPoint(drawRect.Width()-sw-wl-5, start.iY));
					} else {
						int yp = up?sw:drawRect.Height()-sw;
						graphicsContext.DrawTextVertical(strversenum,  TPoint(start.iX, yp), up);
						graphicsContext.SetPenColor(textColor);
						yp = up?sw+wl+5:drawRect.Height()-(sw+wl+5);
						graphicsContext.DrawTextVertical(v->lines->At(iidx), TPoint(start.iX, yp), up);
					}
				} else {
					if (iOrientation==EOrientationNormal) {
						int pos = drawRect.Width() - font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));
						graphicsContext.DrawText((TPtrC16)v->lines->At(iidx), TPoint(pos, start.iY));
					} else {
						if (up) {
							int pos = font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));
							graphicsContext.DrawTextVertical((TPtrC16)v->lines->At(iidx), TPoint(start.iX, pos), up);	
						} else {
							int pos = drawRect.Height() - font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));
							graphicsContext.DrawTextVertical((TPtrC16)v->lines->At(iidx), TPoint(start.iX, pos), up);	
							
						}
					}

				}

				if (iOrientation==EOrientationNormal) {				
					start.iY += font->HeightInPixels();
					inlimit = start.iY<drawRect.Height();
				} else {
						if (up) {
							start.iX += font->HeightInPixels();
							inlimit = start.iX<drawRect.Width();
						} else {
							start.iX -= font->HeightInPixels();
							inlimit = start.iX>0;
						}
				}
				iidx++;
			}
			iidx = 0;
			v = v->next;
		}
	}
	graphicsContext.DiscardFont();	windowgc.BitBlt(Rect().iTl,iOffScreenBitmap);

}

void CS80BibleView::SizeChanged()
{
	//TPoint t = Rect().iTl;
	delete iOffScreenBitmapGc;
	iOffScreenBitmapGc = NULL;

	delete iOffScreenBitmapDevice;
	iOffScreenBitmapDevice = NULL;

	delete iOffScreenBitmap;
	iOffScreenBitmap = NULL;

	iOffScreenBitmap = NBitmapMethods::CreateBitmapL(Rect().Size(),KColourDepth);
	iOffScreenBitmapDevice = NBitmapMethods::CreateBitmapDeviceL(*iOffScreenBitmap);
	iOffScreenBitmapGc = NBitmapMethods::CreateGraphicsContextL(*iOffScreenBitmapDevice);
	DrawDeferred();
	
}

void CS80BibleView::ExecuteCommand(int idx)
{
	/*adjust these according to s60bible.rss*/
	int commands[] =
{
	0,
	KPrevLine,
	KNextLine,
	KPrevPage,
	KNextPage,
	KPrevVerse,
	KNextVerse,
	KPrevChapter,
	KNextChapter,
	KPrevBook,
	KNextBook,
	KGotoBookmark,
	KShowHistory,
	KBookList,
	KGotoTop,
	KGotoBottom,
	KFullscreen
};
	HandleCommandL(commands[idx]);
}


TKeyResponse CS80BibleView::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
	if(aType != EEventKey)
		{
		return EKeyWasNotConsumed;
		}

	/*some special keys*/
	if (aKeyEvent.iScanCode==EStdKeyUpArrow || aKeyEvent.iScanCode==EStdKeyDevice8){
		int x = iDocument->Model()->GetBinding(10);
		if (x!=0) {
			ExecuteCommand(x);
		} else {
			iDocument->Model()->PageUp();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	
	if (aKeyEvent.iScanCode==EStdKeyDownArrow || aKeyEvent.iScanCode==EStdKeyDevice9) {
		int x = iDocument->Model()->GetBinding(11);
		if (x) {
			ExecuteCommand(x);
		} else {
			iDocument->Model()->PageDown();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	
	if (aKeyEvent.iScanCode==EStdKeyLeftArrow ||aKeyEvent.iScanCode==EStdKeyDevice6) {
		int x = iDocument->Model()->GetBinding(12);
		if (x) {
			ExecuteCommand(x);
		} else {
			iDocument->Model()->ScrollUp(1);
			iDocument->Model()->UpdateBuffer();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	if (aKeyEvent.iScanCode==EStdKeyRightArrow ||aKeyEvent.iScanCode==EStdKeyDevice7) {
		int x = iDocument->Model()->GetBinding(13);
		if (x) {
			ExecuteCommand(x);
		} else {
			
			iDocument->Model()->ScrollDown(1);
			iDocument->Model()->UpdateBuffer();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}
