/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/



/*
 * Filename: PDBAccess.cpp
 * Description: PDB (Palm Database) Access Class
 * Author: Yohanes Nugroho
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */
#include <e32std.h>
#include <w32std.h>
//#include <stdio.h>
#include "PDBAccess.h"
#include "util.h"

/**
 * Default constructor
 */
EXPORT_C PDBAccess::PDBAccess()
{
	records = 0;
}

/**
 * Open PDB File
 */
EXPORT_C void PDBAccess::ConstructL(RFs &fsSession, const TDesC &filename)
{
	char *head;
#ifdef __WINS__
	_LIT(KFileName, "z:\\system\\apps\\S60BIBLE\\tb.pdb");
#else
	_LIT(KFileName, "e:\\system\\apps\\S60BIBLE\\tb.pdb");
#endif
//	TAny  *test = User::AllocLC(10);



	records = 0;
	if (filename.Length()==0) {
		User::LeaveIfError(file.Open(fsSession, KFileName, EFileShareReadersOnly));
	} else {
		User::LeaveIfError(file.Open(fsSession, filename, EFileShareReadersOnly));
	}


	file.Size(filesize);

	Mem::FillZ(&pdbheader, sizeof (struct pdb_header));

	TBuf8 <78> buff;

	TInt filestart = 0;
	file.Seek(ESeekStart, filestart);



	int err = file.Read(buff, 78);
	if (err!=KErrNone) {
		file.Close();
		User::Leave(err);
	}

	//Mem::Copy((void *)&pdbheader, (void *)buff.Ptr(), 78);
	head = (char *)buff.Ptr();
	Mem::Copy(pdbheader.name, head, 32);
	Mem::Copy(&pdbheader.attributes, head+32, 2);
	Mem::Copy(&pdbheader.version, head+34, 2);
	Mem::Copy(&pdbheader.create_time, head+36, 4);
	Mem::Copy(&pdbheader.modify_time, head+ 40, 4);
	Mem::Copy(&pdbheader.backup_time, head+44, 4);
	Mem::Copy(&pdbheader.modificationNumber, head+48, 4);
	Mem::Copy(&pdbheader.appInfoID, head+52, 4);
	Mem::Copy(&pdbheader.sortInfoID, head+56, 4);
	Mem::Copy(pdbheader.type, head+60, 4);
	Mem::Copy(pdbheader.creator, head+ 64, 4);
	Mem::Copy(&pdbheader.id_seed, head+68, 4);
	Mem::Copy(&pdbheader.nextRecordList, head+72, 4);
	Mem::Copy(&pdbheader.numRecords, head+76 , 2);

	recordcount = fixs(pdbheader.numRecords);

	records = (pdb_rec_header *)User::AllocL(sizeof(struct pdb_rec_header)*
			recordcount);
	Mem::FillZ(records, sizeof(struct pdb_rec_header)*recordcount);
	//TBuf8 <sizeof(struct pdb_rec_header)> buff2;

/*    	for(int i = 0; i<recordcount; i++){
		TPtr8 tmp((TUint8*)&records[i],sizeof(struct pdb_rec_header));
		file.Read(tmp, sizeof(struct pdb_rec_header));
	 	//file.Read(buff2, sizeof(struct pdb_rec_header));
	 	//Mem::Copy((void *)&records[i], (void *)buff2.Ptr(), sizeof(struct pdb_rec_header));
	 	records[i].offset = fixl(records[i].offset);
	}*/
//	CleanupStack::PopAndDestroy(test);
}


/**
 * Read a record from file
 * @param recno record number
 * @returns record content
 */
char* PDBAccess::readRecord(int recno)
{
	int size = 0, pos=0;
	if (records[recno].offset==0) {
                pos = 78+recno*sizeof(struct pdb_rec_header);
		file.Seek(ESeekStart, pos);
		TPtr8 tmp((TUint8*)&records[recno],sizeof(struct pdb_rec_header));
		file.Read(tmp, sizeof(struct pdb_rec_header));
		records[recno].offset = fixl(records[recno].offset);
	}
        if ((recno<recordcount-1) && (records[recno+1].offset==0)) {
                pos = 78+(recno+1)*sizeof(struct pdb_rec_header);
                file.Seek(ESeekStart, pos);
                TPtr8 tmp2((TUint8*)&records[recno+1],sizeof(struct pdb_rec_header));
                file.Read(tmp2, sizeof(struct pdb_rec_header));
                records[recno+1].offset = fixl(records[recno+1].offset);
        }


	if (recno<recordcount-1) {
		size = 	records[recno+1].offset -
			records[recno].offset;
	} else {
		size = filesize - records[recno].offset;
	}
	TInt offset = records[recno].offset;
	if (file.Seek(ESeekStart, offset)!=KErrNone) {

	}
	char *content = (char *)User::Alloc(size);
	TPtr8 tmp((TUint8*)content,size);
	file.Read(tmp, size);

	/*
	HBufC8* buff;
	buff  = HBufC8::NewL(size);
	TPtr8 ptr = buff->Des();

	file.Read(ptr, size);

	char *content = (char *)User::Alloc(size);
	Mem::Copy((void *)content, (void *)buff->Ptr(), size);
	delete buff;
	*/

	return content;
}

EXPORT_C PDBAccess::~PDBAccess()
{
	if (records) User::Free(records);
	file.Close();
}
